from datetime import datetime
from typing import Optional

from selenium.common import TimeoutException
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import create_webdriver
from uk_bin_collection.uk_bin_collection.common import date_format
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


def wait_for_element(driver, element_type, element: str, timeout: int = 5):
    element_present = EC.presence_of_element_located((element_type, element))
    wait_for_element_conditions(driver, element_present, timeout=timeout)


def wait_for_element_conditions(driver, conditions, timeout: int = 5):
    try:
        WebDriverWait(driver, timeout).until(conditions)
    except TimeoutException:
        print("Timed out waiting for page to load")
        raise


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def __init__(self):
        self._driver: Optional[WebDriver] = None

    def parse_data(self, *args, **kwargs) -> dict:
        try:
            return self._parse_data(*args, **kwargs)
        finally:
            if self._driver:
                self._driver.quit()

    def _parse_data(self, page: str, **kwargs) -> dict:
        """
        Process:

        - Use a house number and postcode that is known to be domestic and resolves to a
          single unique address. When the address search form is submitted with
          those details, a session is created

        - Now a session exists, navigate to the calendar URL, specifying the UPRN

        - Extract info from the 'alt' attribute of the images on that page
        """
        bins = []

        user_paon = kwargs["paon"]
        user_postcode = kwargs["postcode"]

        self._driver = driver = create_webdriver(
            web_driver=kwargs["web_driver"],
            headless=kwargs.get("headless", True),
            session_name=__name__,
        )
        driver.implicitly_wait(1)

        driver.get(
            "https://www.kirklees.gov.uk/beta/your-property-bins-recycling/your-bins/default.aspx"
        )

        wait_for_element(
            driver, By.ID, "cphPageBody_cphContent_thisGeoSearch_txtGeoPremises"
        )

        house_input = driver.find_element(
            By.ID, "cphPageBody_cphContent_thisGeoSearch_txtGeoPremises"
        )
        house_input.send_keys(user_paon)

        postcode_input = driver.find_element(
            By.ID, "cphPageBody_cphContent_thisGeoSearch_txtGeoSearch"
        )
        postcode_input.send_keys(user_postcode)

        # submit address search
        driver.find_element(By.ID, "butGeoSearch").send_keys(Keys.RETURN)

        wait_for_element(
            driver,
            By.ID,
            "cphPageBody_cphContent_wtcDomestic240__lnkAccordionAnchor",
            # submitting can be slow
            timeout=30,
        )

        # Open the panel
        driver.find_element(
            By.ID, "cphPageBody_cphContent_wtcDomestic240__lnkAccordionAnchor"
        ).click()

        # Domestic waste calendar
        wait_for_element(
            driver, By.ID, "cphPageBody_cphContent_wtcDomestic240__LnkCalendar"
        )
        calendar_link = driver.find_element(
            By.ID, "cphPageBody_cphContent_wtcDomestic240__LnkCalendar"
        )
        driver.execute_script("arguments[0].click();", calendar_link)

        # <img alt="Recycling                      collection date 14 March 2024"
        # <img alt="Domestic                       collection date 21 March 2024
        date_strings = driver.find_elements(
            By.CSS_SELECTOR, 'img[alt*="collection date"]'
        )

        for date in date_strings:
            bin_type, _, _, day, month, year = date.get_attribute("alt").split()
            collection_date = datetime.strptime(
                f"{day} {month} {year}", "%d %B %Y"
            ).strftime(date_format)

            bins.append(
                {
                    "type": bin_type,
                    "collectionDate": collection_date,
                }
            )

        return {"bins": bins}



import logging
import time

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# Set up logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            collections = []
            user_uprn = kwargs.get("uprn")
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            if headless:
                driver.set_window_size(1920, 1080)

            driver.get(
                "https://www.cheshirewestandchester.gov.uk/residents/waste-and-recycling/your-bin-collection/collection-day"
            )
            wait = WebDriverWait(driver, 60)

            def click_element(by, value):
                element = wait.until(EC.element_to_be_clickable((by, value)))
                driver.execute_script("arguments[0].scrollIntoView();", element)
                element.click()

            logging.info("Accepting cookies")
            click_element(By.ID, "ccc-close")

            logging.info("Finding collection day")
            click_element(By.LINK_TEXT, "Find your collection day")

            logging.info("Switching to iframe")
            iframe_presence = wait.until(
                EC.presence_of_element_located((By.ID, "fillform-frame-1"))
            )
            driver.switch_to.frame(iframe_presence)

            logging.info("Entering postcode")
            input_element_postcode = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, '//input[@id="postcode_search"]')
                )
            )
            input_element_postcode.send_keys(user_postcode)

            pcsearch_btn = wait.until(
                EC.element_to_be_clickable((By.XPATH, "//input[@id='postcode_search']"))
            )
            click_element(By.XPATH, "//input[@id='postcode_search']")

            logging.info("Selecting address")
            dropdown = wait.until(EC.element_to_be_clickable((By.ID, "Choose_Address")))
            dropdown_options = wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "lookup-option"))
            )
            drop_down_values = Select(dropdown)
            option_element = wait.until(
                EC.presence_of_element_located(
                    (By.CSS_SELECTOR, f'option.lookup-option[value="{str(user_uprn)}"]')
                )
            )
            driver.execute_script("arguments[0].scrollIntoView();", option_element)
            drop_down_values.select_by_value(str(user_uprn))

            logging.info("Waiting for bin schedule")
            wait.until(
                EC.presence_of_element_located(
                    (By.CLASS_NAME, "bin-schedule-content-bin-card")
                )
            )

            logging.info("Extracting bin collection data")
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            bin_cards = soup.find_all("div", {"class": "bin-schedule-content-bin-card"})
            collections = []

            for card in bin_cards:
                bin_info = card.find("div", {"class": "bin-schedule-content-info"})
                bin_name = bin_info.find_all("p")[0].text.strip() + " bin"
                bin_date_str = bin_info.find_all("p")[1].text.split(":")[1].strip()
                bin_date = datetime.strptime(bin_date_str, "%A, %B %d, %Y")
                collections.append((bin_name, bin_date))

            ordered_data = sorted(collections, key=lambda x: x[1])

            for item in ordered_data:
                dict_data = {
                    "type": item[0].capitalize(),
                    "collectionDate": item[1].strftime(date_format),
                }
                data["bins"].append(dict_data)

            logging.info("Data extraction complete")
            return data

        except Exception as e:
            logging.error(f"An error occurred: {e}")
            raise

        finally:
            if driver:
                driver.quit()



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        api_url = f"https://geoserver.nottinghamcity.gov.uk/myproperty/handler/proxy.ashx?https://geoserver.nottinghamcity.gov.uk/bincollections2/api/collection/{user_uprn}"

        requests.packages.urllib3.disable_warnings()
        response = requests.get(api_url)
        json_data = json.loads(response.text)
        data = {"bins": []}

        next_collections = json_data["nextCollections"]

        for collection in next_collections:
            bin_type = collection["collectionType"]

            next_collection_date = datetime.fromisoformat(collection["collectionDate"])
            dict_data = {
                "type": bin_type,
                "collectionDate": next_collection_date.strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        url_base = "https://basildonportal.azurewebsites.net/api/getPropertyRefuseInformation"

        uprn = kwargs.get("uprn")
        # Check the UPRN is valid
        check_uprn(uprn)

        payload = {
            # Add your payload details here (replace this with the actual payload structure if required)
            "uprn": uprn
        }

        # Headers for the request
        headers = {
            "Content-Type": "application/json"
        }

        response = requests.post(url_base, data=json.dumps(payload), headers=headers)

        # Ensure the request was successful
        if response.status_code == 200:
            data = response.json()

            # Initialize an empty list to store the bin collection details

            bins = []

            # Function to add collection details to bins list
            def add_collection(service_name, collection_data):
                bins.append({
                    "type": service_name,
                    "collectionDate": collection_data.get("current_collection_date")
                })

            # Extract refuse information
            available_services = data["refuse"]["available_services"]

            for service_name, service_data in available_services.items():
                # Append the service name and current collection date to the "bins" list
                match service_data["container"]:
                    case "Green Wheelie Bin":
                        subscription_status = service_data["subscription"]["active"] if service_data["subscription"] else False
                        type_descr = f"Green Wheelie Bin ({"Active" if subscription_status else "Expired"})"
                    case "N/A":
                        type_descr = service_data["name"]
                    case _:
                        type_descr = service_data["container"]


                date_str = service_data.get("current_collection_date")
                # Parse the date string into a datetime object
                date_obj = datetime.strptime(date_str, "%Y-%m-%d")

                # Convert the datetime object to the desired format
                formatted_date = date_obj.strftime(date_format)

                bins.append({
                    "type": type_descr,  # Use service name from the data
                    "collectionDate": formatted_date
                })

        else:
            print(f"Failed to fetch data. Status code: {response.status_code}")

        data = {
            "bins": bins
        }

        return data



import time
from datetime import datetime

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}

            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            # Create Selenium webdriver
            page = (
                f"https://webapp.halton.gov.uk/PublicWebForms/WasteServiceSearchv1.aspx"
            )

            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            # If you bang in the house number (or property name) and postcode in the box it should find your property

            # iframe_presense = WebDriverWait(driver, 30).until(
            #    EC.presence_of_element_located((By.ID, "fillform-frame-1"))
            # )

            # driver.switch_to.frame(iframe_presense)
            wait = WebDriverWait(driver, 60)

            inputElement_property = wait.until(
                EC.element_to_be_clickable(
                    (By.NAME, "ctl00$ContentPlaceHolder1$txtProperty")
                )
            )
            inputElement_property.send_keys(user_paon)

            inputElement_postcodesearch = wait.until(
                EC.element_to_be_clickable(
                    (By.NAME, "ctl00$ContentPlaceHolder1$txtPostcode")
                )
            )
            inputElement_postcodesearch.send_keys(user_postcode)
            time.sleep(1)
            wait.until(
                EC.frame_to_be_available_and_switch_to_it(
                    (
                        By.CSS_SELECTOR,
                        "iframe[name^='a-'][src^='https://www.google.com/recaptcha/api2/anchor?']",
                    )
                )
            )
            wait.until(
                EC.element_to_be_clickable((By.XPATH, "//span[@id='recaptcha-anchor']"))
            ).send_keys(Keys.ENTER)
            time.sleep(5)
            driver.switch_to.default_content()
            search_btn = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, '//*[@id="ContentPlaceHolder1_btnSearch"]')
                )
            )
            search_btn.send_keys(Keys.ENTER)
            soup = BeautifulSoup(driver.page_source, features="html.parser")

            # Find all tab panels within the collectionTabs
            # Find all anchor elements within the collectionTabs
            anchor_elements = soup.select("#collectionTabs a.ui-tabs-anchor")

            for anchor in anchor_elements:
                # Extract the type of waste from the anchor text
                waste_type = anchor.text.strip()

                # Find the corresponding panel using the href attribute
                panel_id = anchor.get("href")
                panel = soup.select_one(panel_id)

                # Find all ul elements within the corresponding panel
                ul_elements = panel.find_all("ul")

                # Check if there are at least two ul elements
                if len(ul_elements) >= 2:
                    # Get the second ul element and extract its li elements
                    second_ul = ul_elements[1]
                    li_elements = second_ul.find_all("li")

                    # Extract the text content of each li element
                    date_texts = [
                        re.sub(r"[^a-zA-Z0-9,\s]", "", li.get_text(strip=True)).strip()
                        for li in li_elements
                    ]

                    for date_text in date_texts:
                        # Extracting dates from the text using simple text manipulation
                        # Assuming the dates are in the format: "Friday 15th December 2023", "Friday 22nd December 2023", etc.
                        # Parse the date string into a datetime object
                        date_string_without_ordinal = re.sub(
                            r"(\d+)(st|nd|rd|th)", r"\1", date_text
                        )

                        parsed_date = datetime.strptime(
                            date_string_without_ordinal, "%A %d %B %Y"
                        )

                        # Format the datetime object into the desired format '%d/%m/%Y'
                        formatted_date = parsed_date.strftime("%d/%m/%Y")

                        # Add extracted data to the 'bins' list
                        data["bins"].append(
                            {
                                "type": waste_type.capitalize(),
                                "collectionDate": formatted_date,
                            }
                        )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from typing import Dict, Any
from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta
import requests
from datetime import datetime
from uk_bin_collection.uk_bin_collection.common import (
    check_postcode,
    check_uprn,
    date_format,
)
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs: Any) -> Dict[str, Any]:
        data: Dict[str, Any] = {"bins": []}

        # Get UPRN and postcode from kwargs
        user_uprn = str(kwargs.get("uprn"))
        user_postcode = str(kwargs.get("postcode"))
        check_postcode(user_postcode)
        check_uprn(user_uprn)

        # Pass in form data and make the POST request
        headers = {
            "authority": "waste.barnsley.gov.uk",
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "accept-language": "en-GB,en;q=0.9",
            "cache-control": "no-cache",
            "content-type": "application/x-www-form-urlencoded",
            "origin": "https://waste.barnsley.gov.uk",
            "pragma": "no-cache",
            "referer": "https://waste.barnsley.gov.uk/ViewCollection/SelectAddress",
            "sec-ch-ua": '"Chromium";v="118", "Opera GX";v="104", "Not=A?Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
            "sec-fetch-dest": "document",
            "sec-fetch-mode": "navigate",
            "sec-fetch-site": "same-origin",
            "sec-fetch-user": "?1",
            "upgrade-insecure-requests": "1",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.5993.118 Safari/537.36",
        }
        form_data = {
            "personInfo.person1.HouseNumberOrName": "",
            "personInfo.person1.Postcode": f"{user_postcode}",
            "personInfo.person1.UPRN": f"{user_uprn}",
            "person1_SelectAddress": "Select address",
        }
        response = requests.post(
            "https://waste.barnsley.gov.uk/ViewCollection/SelectAddress",
            headers=headers,
            data=form_data,
        )

        if response.status_code != 200:
            raise ConnectionRefusedError(
                "Error getting results from website! Please open an issue on GitHub!"
            )

        soup = BeautifulSoup(response.text, features="html.parser")

        results = soup.find_all("fieldset")

        # Next collection details
        highlight_content = results[0].find("div", {"class": "highlight-content"})
        bin_date_str = highlight_content.find(
            "em", {"class": "ui-bin-next-date"}
        ).text.strip()
        bin_type = (
            highlight_content.find("p", {"class": "ui-bin-next-type"}).text.strip()
            + " bin"
        )

        if bin_date_str == "Today":
            bin_date = datetime.today()
        elif bin_date_str == "Tomorrow":
            bin_date = datetime.today() + relativedelta(days=1)
        else:
            bin_date = datetime.strptime(bin_date_str, "%A, %B %d, %Y")

        dict_data = {
            "type": bin_type,
            "collectionDate": bin_date.strftime(date_format),
        }
        data["bins"].append(dict_data)

        # Upcoming collections
        upcoming_collections = results[1].find("tbody").find_all("tr")
        for row in upcoming_collections:
            columns = row.find_all("td")
            bin_date_str = columns[0].text.strip()
            bin_types = columns[1].text.strip().split(", ")

            for bin_type in bin_types:
                bin_date = datetime.strptime(bin_date_str, "%A, %B %d, %Y")
                dict_data = {
                    "type": bin_type.strip() + " bin",
                    "collectionDate": bin_date.strftime(date_format),
                }
                data["bins"].append(dict_data)

        return data



import json
from datetime import datetime

import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


def get_address_uprn(postcode: str, paon: str, api_url: str) -> str:
    """
    Gets the UPRN and address in desired format
        :rtype: str
        :param postcode: Postcode to use
        :param paon: House number to use
        :param api_url: API to POST
        :return: UPRN and postcode in str format
    """
    addr = ""
    payload = json.dumps(
        {
            "jsonrpc": "2.0",
            "id": "1689431267990",
            "method": "stc.common.snippets.getAddressList",
            "params": {"postcode": f"{postcode.replace(' ', '')}", "localonly": "true"},
        }
    )
    headers = {"Content-Type": "application/json"}
    response = requests.post(api_url, data=payload, headers=headers)

    json_response = json.loads(response.content)
    results = json_response["result"]["ReturnedList"]

    for item in results:
        if item["Address"].split()[0] == paon.strip():
            addr = item["UPRN"] + "|" + item["Address"]
            break

    return addr


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        requests.packages.urllib3.disable_warnings()
        api_url = "https://www.southtyneside.gov.uk/apiserver/ajaxlibrary/"
        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")
        data = {"bins": []}

        check_postcode(user_postcode)
        check_paon(user_paon)

        try:
            if user_paon is None:
                raise ValueError("Invalid house number")
        except Exception as ex:
            print(f"Exception encountered: {ex}")
            print(
                "Please check the provided house number. If this error continues, please first trying setting the "
                "house number manually on line 25 before raising an issue."
            )
            exit(1)

        # Get the "UPRN" (actually the UPRN + address)
        uprn = get_address_uprn(user_postcode, user_paon, api_url)

        # Set up payload and headers, then post to API to get schedule
        payload = json.dumps(
            {
                "jsonrpc": "2.0",
                "id": "1689431609779",
                "method": "stc.waste.collections.getDates",
                "params": {"addresscode": uprn},
            }
        )
        headers = {"Content-Type": "application/json"}
        response = requests.request("POST", api_url, headers=headers, data=payload)

        # Break down the resulting JSON and load into dictionary
        json_result = json.loads(response.text)["result"]
        months = json_result["SortedCollections"]
        for month in months:
            collections_in_month = month["Collections"]
            for item in collections_in_month:
                dict_data = {
                    "type": item["Type"],
                    "collectionDate": datetime.strptime(
                        item["DateString"], "%d %B %Y"
                    ).strftime(date_format),
                }
                data["bins"].append(dict_data)

        return data



from unittest.mock import MagicMock, patch
import argparse
import pytest
from uk_bin_collection.collect_data import UKBinCollectionApp, import_council_module



# Test UKBinCollectionApp setup_arg_parser
def test_setup_arg_parser():
    app = UKBinCollectionApp()
    app.setup_arg_parser()

    # Assert that the argument parser has the correct arguments
    assert isinstance(app.parser, argparse.ArgumentParser)
    args = app.parser._actions
    arg_names = [action.dest for action in args]

    expected_args = [
        "module",
        "URL",
        "postcode",
        "number",
        "skip_get_url",
        "uprn",
        "web_driver",
        "headless",
        "local_browser",
        "dev_mode",
    ]
    assert all(arg in arg_names for arg in expected_args)


# Test UKBinCollectionApp set_args
def test_set_args():
    app = UKBinCollectionApp()
    app.setup_arg_parser()

    # Test valid args
    args = ["council_module", "http://example.com", "--postcode", "AB1 2CD"]
    app.set_args(args)

    assert app.parsed_args.module == "council_module"
    assert app.parsed_args.URL == "http://example.com"
    assert app.parsed_args.postcode == "AB1 2CD"


# Test UKBinCollectionApp client_code method
def test_client_code():
    app = UKBinCollectionApp()
    mock_get_bin_data_class = MagicMock()

    # Run the client_code and ensure that template_method is called
    app.client_code(mock_get_bin_data_class, "http://example.com", postcode="AB1 2CD")
    mock_get_bin_data_class.template_method.assert_called_once_with(
        "http://example.com", postcode="AB1 2CD"
    )


# Test the run() function with logging setup
@patch("uk_bin_collection.collect_data.setup_logging")  # Correct patch path
@patch("uk_bin_collection.collect_data.UKBinCollectionApp.run")  # Correct patch path
@patch("sys.argv", ["uk_bin_collection.py", "council_module", "http://example.com"])
def test_run_function(mock_app_run, mock_setup_logging):
    from uk_bin_collection.collect_data import run

    mock_setup_logging.return_value = MagicMock()
    mock_app_run.return_value = None

    run()

    # Ensure logging was set up and the app run method was called
    mock_setup_logging.assert_called_once()
    mock_app_run.assert_called_once()



import logging
import time

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# Set up logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete class for Mid-Sussex District Council implementing AbstractGetBinDataClass.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            collections = []
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)

            driver.get("https://www.midsussex.gov.uk/waste-recycling/bin-collection/")
            wait = WebDriverWait(driver, 60)

            try:
                logging.info("Cookies")
                cookie_window = wait.until(
                    EC.presence_of_element_located(
                        (By.XPATH, '//div[@id="ccc-content"]')
                    )
                )
                time.sleep(2)
                accept_cookies = WebDriverWait(driver, timeout=10).until(
                    EC.presence_of_element_located(
                        (By.XPATH, '//button[@id="ccc-recommended-settings"]')
                    )
                )
                accept_cookies.send_keys(Keys.ENTER)
                accept_cookies.click()
                accept_cookies_close = WebDriverWait(driver, timeout=10).until(
                    EC.presence_of_element_located(
                        (By.XPATH, '//button[@id="ccc-close"]')
                    )
                )
                accept_cookies_close.send_keys(Keys.ENTER)
                accept_cookies_close.click()
            except:
                print(
                    "Accept cookies banner not found or clickable within the specified time."
                )
                pass

            def click_element(by, value):
                element = wait.until(EC.element_to_be_clickable((by, value)))
                driver.execute_script("arguments[0].scrollIntoView();", element)
                element.click()

            logging.info("Entering postcode")
            input_element_postcode = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, '//input[@id="PostCodeStep_strAddressSearch"]')
                )
            )

            input_element_postcode.send_keys(user_postcode)

            logging.info("Entering postcode")

            click_element(By.XPATH, "//button[contains(text(), 'Search')]")

            logging.info("Selecting address")
            dropdown = wait.until(
                EC.element_to_be_clickable((By.ID, "StrAddressSelect"))
            )

            dropdown_options = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, "//select[@id='StrAddressSelect']/option")
                )
            )
            dropdownSelect = Select(dropdown)
            dropdownSelect.select_by_visible_text(str(user_paon))

            click_element(By.XPATH, "//button[contains(text(), 'Select')]")

            logging.info("Waiting for bin schedule")
            bin_results = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f"//strong[contains(text(), '{user_paon}')]")
                )
            )

            # Make a BS4 object
            soup = BeautifulSoup(driver.page_source, features="html.parser")

            # Find the table with bin collection data
            table = soup.find("table", class_="collDates")
            if table:
                rows = table.find_all("tr")[1:]  # Skip the header row
            else:
                rows = []

            # Extract the data from the table and format it according to the JSON schema
            bins = []
            date_pattern = re.compile(r"(\d{2}) (\w+) (\d{4})")

            for row in rows:
                cols = row.find_all("td")
                if len(cols) < 3:
                    print("Skipping row, not enough columns:", row)
                    continue  # Skip rows that do not have enough columns

                collection_type = cols[1].text.strip()
                collection_date = cols[2].text.strip()

                # Convert the collection date to the required format
                date_match = date_pattern.search(collection_date)
                if date_match:
                    day, month, year = date_match.groups()
                    month_number = {
                        "January": "01",
                        "February": "02",
                        "March": "03",
                        "April": "04",
                        "May": "05",
                        "June": "06",
                        "July": "07",
                        "August": "08",
                        "September": "09",
                        "October": "10",
                        "November": "11",
                        "December": "12",
                    }.get(month, "00")

                    formatted_date = f"{day}/{month_number}/{year}"
                    bins.append(
                        {"type": collection_type, "collectionDate": formatted_date}
                    )
                else:
                    print("Date pattern not found in:", collection_date)

            # Create the final JSON structure
            bin_data = {"bins": bins}
            return bin_data
        except Exception as e:
            logging.error(f"An error occurred: {e}")
            raise

        finally:
            if driver:
                driver.quit()



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")

        check_uprn(user_uprn)

        response = requests.post(
            "https://www.rotherham.gov.uk/bin-collections?address={}&submit=Submit".format(user_uprn)
        )
        # Make a BS4 object
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        table = soup.select('table')[0]

        if table:
            rows = table.select('tr')

            for index, row in enumerate(rows):
                bin_info_cell = row.select('td')
                if bin_info_cell: 
                    bin_type = bin_info_cell[0].get_text(separator=' ', strip=True)
                    bin_collection = bin_info_cell[1]

                    if bin_collection:
                        dict_data = {
                            "type": bin_type.title(),
                            "collectionDate": datetime.strptime(
                                bin_collection.get_text(strip=True), "%A, %d %B %Y"
                            ).strftime(date_format),
                        }

                    data["bins"].append(dict_data)
        else:
            print("Something went wrong. Please open a GitHub issue.")

        return data



import time
from typing import Any

from dateutil.parser import parse
from selenium.common.exceptions import NoSuchElementException, TimeoutException
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select, WebDriverWait

from uk_bin_collection.uk_bin_collection.common import create_webdriver, date_format
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    def wait_for_element_conditions(self, driver, conditions, timeout: int = 5):
        try:
            WebDriverWait(driver, timeout).until(conditions)
        except TimeoutException:
            print("Timed out waiting for page to load")
            raise

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            page = "https://sevenoaks-dc-host01.oncreate.app/w/webpage/waste-collection-day"

            # Assign user info
            user_postcode = kwargs.get("postcode")
            user_paon = kwargs.get("paon")

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            # Enter postcode
            postcode_css_selector = "#address_search_postcode"
            self.wait_for_element_conditions(
                driver,
                EC.presence_of_element_located(
                    (By.CSS_SELECTOR, postcode_css_selector)
                ),
            )
            postcode_input_box = driver.find_element(
                By.CSS_SELECTOR, postcode_css_selector
            )
            postcode_input_box.send_keys(user_postcode)
            postcode_input_box.send_keys(Keys.ENTER)

            # Select the dropdown
            self.wait_for_element_conditions(
                driver, EC.presence_of_element_located((By.XPATH, "//select/option[2]"))
            )
            select_address_dropdown = Select(driver.find_element(By.XPATH, "//select"))

            if user_paon is not None:
                for option in select_address_dropdown.options:
                    if user_paon in option.text:
                        select_address_dropdown.select_by_visible_text(option.text)
                        break
            else:
                # If we've not been supplied an address, pick the second entry
                select_address_dropdown.select_by_index(1)

            # Grab the response blocks
            response_xpath_selector = "//div[@data-class_name]//h4/../../../.."
            self.wait_for_element_conditions(
                driver,
                EC.presence_of_element_located((By.XPATH, response_xpath_selector)),
            )
            elements = driver.find_elements(By.XPATH, response_xpath_selector)

            # Iterate through them
            data = {"bins": []}
            for element in elements:
                try:
                    raw_bin_name = element.find_element(By.TAG_NAME, "h4").text
                    raw_next_collection_date = element.find_elements(
                        By.XPATH, ".//div[input]"
                    )[1].text

                    parsed_bin_date = parse(
                        raw_next_collection_date, fuzzy_with_tokens=True
                    )[0]

                    dict_data = {
                        "type": raw_bin_name,
                        "collectionDate": parsed_bin_date.strftime(date_format),
                    }

                    data["bins"].append(dict_data)

                except (IndexError, NoSuchElementException):
                    print("Error finding element for bin")
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data
